@@ -1,4 +1,4 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2003 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::MainWindow;
@@ -8,7 +8,7 @@ BEGIN { @MainWindow::ISA = 'Tk::MainWindow' }
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '4.005'; # $Id: //depot/Tkutf8/Tk/MainWindow.pm#5 $
+$VERSION = '4.013'; # was: sprintf '4.%03d', q$Revision: #12 $ =~ /\D(\d+)\s*$/;
 
 use Tk::CmdLine;
 use Tk qw(catch);
@@ -81,19 +81,24 @@ sub InitBindings
 {
  my $mw = shift;
  $mw->bind('all','<Tab>','focusNext');
+ # <<LeftTab>> is named <<PrevWindow>> in Tcl/Tk
  $mw->eventAdd(qw[<<LeftTab>> <Shift-Tab>]);
- catch {  $mw->eventAdd(qw[<<LeftTab>> <ISO_Left_Tab>]) };
+ # This is needed for XFree86 systems
+ catch { $mw->eventAdd(qw[<<LeftTab>> <ISO_Left_Tab>]) };
+ # This seems to be correct on *some* HP systems.
+ catch { $mw->eventAdd(qw[<<LeftTab>> <hpBackTab>]) };
  $mw->bind('all','<<LeftTab>>','focusPrev');
- if ($Tk::platform eq 'unix')
+ if ($mw->windowingsystem eq 'x11')
   {
    $mw->eventAdd(qw[<<Cut>> <Control-Key-x> <Key-F20> <Meta-Key-w>]);
    $mw->eventAdd(qw[<<Copy>> <Control-Key-c> <Key-F16> <Control-Key-w>]);
    $mw->eventAdd(qw[<<Paste>> <Control-Key-v> <Key-F18> <Control-Key-y>]);
+   $mw->eventAdd(qw[<<PasteSelection>> <ButtonRelease-2>]);
    $mw->eventAdd(qw[<<Undo>> <Control-Key-z> <Key-Undo> <Key-F14>
                     <Control-Key-underscore>]);
    $mw->eventAdd(qw[<<Redo>> <Control-Key-y> <Shift-Key-Undo> <Key-F12> <Shift-Key-F14>]);
   }
- else
+ elsif ($mw->windowingsystem eq 'win32')
   {
    $mw->eventAdd(qw[<<Cut>> <Control-Key-x> <Shift-Key-Delete>]);
    $mw->eventAdd(qw[<<Copy>> <Control-Key-c> <Control-Key-Insert>]);
@@ -101,6 +106,26 @@ sub InitBindings
    $mw->eventAdd(qw[<<Undo>> <Control-Key-z>]);
    $mw->eventAdd(qw[<<Redo>> <Control-Key-y>]);
   }
+ elsif ($mw->windowingsystem eq 'aqua')
+  {
+   $mw->eventAdd(qw[<<Cut>> <Command-Key-x> <Key-F2>]);
+   $mw->eventAdd(qw[<<Copy>> <Command-Key-c> <Key-F3>]);
+   $mw->eventAdd(qw[<<Paste>> <Command-Key-v> <Key-F4>]);
+   $mw->eventAdd(qw[<<PasteSelection>> <ButtonRelease-2>]);
+   $mw->eventAdd(qw[<<Clear>> <Clear>]);
+   $mw->eventAdd(qw[<<Undo>> <Command-Key-z>]);
+   $mw->eventAdd(qw[<<Redo>> <Command-Key-y>]);
+  }
+ elsif ($mw->windowingsystem eq 'classic')
+  {
+   $mw->eventAdd(qw[<<Cut>> <Control-Key-x> <Key-F2>]);
+   $mw->eventAdd(qw[<<Copy>> <Control-Key-c> <Key-F3>]);
+   $mw->eventAdd(qw[<<Paste>> <Control-Key-v> <Key-F4>]);
+   $mw->eventAdd(qw[<<PasteSelection>> <ButtonRelease-2>]);
+   $mw->eventAdd(qw[<<Clear>> <Clear>]);
+   $mw->eventAdd(qw[<<Undo>> <Control-Key-z> <Key-F1>]);
+   $mw->eventAdd(qw[<<Redo>> <Control-Key-Z>]);
+  }
 
  # FIXME - Should these move to Menubutton ?
  my $c = ($Tk::platform eq 'unix') ? 'all' : 'Tk::Menubutton';
@@ -128,7 +153,7 @@ sub Existing
 
 END
 {
- if ($pid == $$)
+ if (Tk::IsParentProcess())
   {
    foreach my $top (values %Windows)
     {
@@ -150,7 +175,7 @@ sub WMSaveYourself
 {
  my $mw  = shift;
  my @args = @{$mw->command};
- warn 'preWMSaveYourself:'.join(' ',@args)."\n";
+# warn 'preWMSaveYourself:'.join(' ',@args)."\n";
  @args = ($0) unless (@args);
  my $i = 1;
  while ($i < @args)
@@ -166,7 +191,7 @@ sub WMSaveYourself
   }
 
  my @ip = $mw->wm('iconposition');
- print 'ip ',join(',',@ip),"\n";
+# print 'ip ',join(',',@ip),"\n";
  my $icon = $mw->iconwindow;
  if (defined($icon))
   {
@@ -177,7 +202,7 @@ sub WMSaveYourself
  splice(@args,1,0,'-iconic') if ($mw->state() eq 'iconic');
 
  splice(@args,1,0,'-geometry' => $mw->geometry);
- warn 'postWMSaveYourself:'.join(' ',@args)."\n";
+# warn 'postWMSaveYourself:'.join(' ',@args)."\n";
  $mw->command([@args]);
 }
 
@@ -185,4 +210,3 @@ sub WMSaveYourself
 
 __END__
 
-=cut
