@@ -1,4 +1,4 @@
-/* 
+/*
  * tkTextMark.c --
  *
  *	This file contains the procedure that implement marks for
@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkTextMark.c,v 1.2 1998/09/14 18:23:19 stanton Exp $
+ * RCS: @(#) $Id: tkTextMark.c,v 1.6 2002/08/05 04:30:40 dgp Exp $
  */
 
 #include "tkInt.h"
@@ -42,9 +42,9 @@ static int		MarkLayoutProc _ANSI_ARGS_((TkText *textPtr,
 			    int noCharsYet, TkWrapMode wrapMode,
 			    TkTextDispChunk *chunkPtr));
 static int		MarkFindNext _ANSI_ARGS_((Tcl_Interp *interp,
-			    TkText *textPtr, char *markName));
+			    TkText *textPtr, CONST char *markName));
 static int		MarkFindPrev _ANSI_ARGS_((Tcl_Interp *interp,
-			    TkText *textPtr, char *markName));
+			    TkText *textPtr, CONST char *markName));
 
 
 /*
@@ -75,7 +75,7 @@ Tk_SegType tkTextLeftMarkType = {
     MarkLayoutProc,				/* layoutProc */
     MarkCheckProc				/* checkProc */
 };
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -134,9 +134,9 @@ TkTextMarkCmd(textPtr, interp, argc, argv)
 	markPtr = (TkTextSegment *) Tcl_GetHashValue(hPtr);
 	if (argc == 4) {
 	    if (markPtr->typePtr == &tkTextRightMarkType) {
-		interp->result = "right";
+		Tcl_SetResult(interp, "right", TCL_STATIC);
 	    } else {
-		interp->result = "left";
+		Tcl_SetResult(interp, "left", TCL_STATIC);
 	    }
 	    return TCL_OK;
 	}
@@ -214,7 +214,7 @@ TkTextMarkCmd(textPtr, interp, argc, argv)
     }
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -235,7 +235,7 @@ TkTextMarkCmd(textPtr, interp, argc, argv)
 TkTextSegment *
 TkTextSetMark(textPtr, name, indexPtr)
     TkText *textPtr;		/* Text widget in which to create mark. */
-    char *name;			/* Name of mark to set. */
+    CONST char *name;			/* Name of mark to set. */
     TkTextIndex *indexPtr;	/* Where to set mark. */
 {
     Tcl_HashEntry *hPtr;
@@ -290,7 +290,7 @@ TkTextSetMark(textPtr, name, indexPtr)
     }
     return markPtr;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -319,13 +319,13 @@ TkTextMarkSegToIndex(textPtr, markPtr, indexPtr)
 
     indexPtr->tree = textPtr->tree;
     indexPtr->linePtr = markPtr->body.mark.linePtr;
-    indexPtr->charIndex = 0;
+    indexPtr->byteIndex = 0;
     for (segPtr = indexPtr->linePtr->segPtr; segPtr != markPtr;
 	    segPtr = segPtr->nextPtr) {
-	indexPtr->charIndex += segPtr->size;
+	indexPtr->byteIndex += segPtr->size;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -350,7 +350,7 @@ TkTextMarkSegToIndex(textPtr, markPtr, indexPtr)
 int
 TkTextMarkNameToIndex(textPtr, name, indexPtr)
     TkText *textPtr;		/* Text widget containing mark. */
-    char *name;			/* Name of mark. */
+    CONST char *name;		/* Name of mark. */
     TkTextIndex *indexPtr;	/* Index information gets stored here. */
 {
     Tcl_HashEntry *hPtr;
@@ -363,7 +363,7 @@ TkTextMarkNameToIndex(textPtr, name, indexPtr)
 	    indexPtr);
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -393,7 +393,7 @@ MarkDeleteProc(segPtr, linePtr, treeGone)
 {
     return 1;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -419,7 +419,7 @@ MarkCleanupProc(markPtr, linePtr)
     markPtr->body.mark.linePtr = linePtr;
     return markPtr;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -468,7 +468,7 @@ MarkLayoutProc(textPtr, indexPtr, segPtr, offset, maxX, maxChars,
     chunkPtr->undisplayProc = InsertUndisplayProc;
     chunkPtr->measureProc = (Tk_ChunkMeasureProc *) NULL;
     chunkPtr->bboxProc = (Tk_ChunkBboxProc *) NULL;
-    chunkPtr->numChars = 0;
+    chunkPtr->numBytes = 0;
     chunkPtr->minAscent = 0;
     chunkPtr->minDescent = 0;
     chunkPtr->minHeight = 0;
@@ -484,7 +484,7 @@ MarkLayoutProc(textPtr, indexPtr, segPtr, offset, maxX, maxChars,
     chunkPtr->clientData = (ClientData) textPtr;
     return 1;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -526,12 +526,16 @@ TkTextInsertDisplayProc(chunkPtr, x, y, height, baseline, display, dst, screenY)
 
     if ((x + halfWidth) < 0) {
 	/*
-	 * The insertion cursor is off-screen.  Just return.
+	 * The insertion cursor is off-screen.
+	 * Indicate caret at 0,0 and return.
 	 */
 
+	Tk_SetCaretPos(textPtr->tkwin, 0, 0, height);
 	return;
     }
 
+    Tk_SetCaretPos(textPtr->tkwin, x - halfWidth, screenY, height);
+
     /*
      * As a special hack to keep the cursor visible on mono displays
      * (or anywhere else that the selection and insertion cursors
@@ -542,15 +546,15 @@ TkTextInsertDisplayProc(chunkPtr, x, y, height, baseline, display, dst, screenY)
 
     if (textPtr->flags & INSERT_ON) {
 	Tk_Fill3DRectangle(textPtr->tkwin, dst, textPtr->insertBorder,
-		x - textPtr->insertWidth/2, y, textPtr->insertWidth,
-		height, textPtr->insertBorderWidth, TK_RELIEF_RAISED);
+		x - halfWidth, y, textPtr->insertWidth, height,
+		textPtr->insertBorderWidth, TK_RELIEF_RAISED);
     } else if (textPtr->selBorder == textPtr->insertBorder) {
 	Tk_Fill3DRectangle(textPtr->tkwin, dst, textPtr->border,
-		x - textPtr->insertWidth/2, y, textPtr->insertWidth,
-		height, 0, TK_RELIEF_FLAT);
+		x - halfWidth, y, textPtr->insertWidth, height,
+		0, TK_RELIEF_FLAT);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -578,7 +582,7 @@ InsertUndisplayProc(textPtr, chunkPtr)
 {
     return;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -622,7 +626,7 @@ MarkCheckProc(markPtr, linePtr)
 	}
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -643,7 +647,7 @@ static int
 MarkFindNext(interp, textPtr, string)
     Tcl_Interp *interp;			/* For error reporting */
     TkText *textPtr;			/* The widget */
-    char *string;			/* The starting index or mark name */
+    CONST char *string;			/* The starting index or mark name */
 {
     TkTextIndex index;
     Tcl_HashEntry *hPtr;
@@ -668,8 +672,8 @@ MarkFindNext(interp, textPtr, string)
 	if (TkTextGetIndex(interp, textPtr, string, &index) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	for (offset = 0, segPtr = index.linePtr->segPtr; 
-		segPtr != NULL && offset < index.charIndex;
+	for (offset = 0, segPtr = index.linePtr->segPtr;
+		segPtr != NULL && offset < index.byteIndex;
 		offset += segPtr->size,	segPtr = segPtr->nextPtr) {
 	    /* Empty loop body */ ;
 	}
@@ -692,11 +696,11 @@ MarkFindNext(interp, textPtr, string)
 	if (index.linePtr == (TkTextLine *) NULL) {
 	    return TCL_OK;
 	}
-	index.charIndex = 0;
+	index.byteIndex = 0;
 	segPtr = index.linePtr->segPtr;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -717,7 +721,7 @@ static int
 MarkFindPrev(interp, textPtr, string)
     Tcl_Interp *interp;			/* For error reporting */
     TkText *textPtr;			/* The widget */
-    char *string;			/* The starting index or mark name */
+    CONST char *string;			/* The starting index or mark name */
 {
     TkTextIndex index;
     Tcl_HashEntry *hPtr;
@@ -741,8 +745,8 @@ MarkFindPrev(interp, textPtr, string)
 	if (TkTextGetIndex(interp, textPtr, string, &index) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	for (offset = 0, segPtr = index.linePtr->segPtr; 
-		segPtr != NULL && offset < index.charIndex;
+	for (offset = 0, segPtr = index.linePtr->segPtr;
+		segPtr != NULL && offset < index.byteIndex;
 		offset += segPtr->size, segPtr = segPtr->nextPtr) {
 	    /* Empty loop body */ ;
 	}
@@ -752,7 +756,7 @@ MarkFindPrev(interp, textPtr, string)
 	 * segPtr points just past the first possible candidate,
 	 * or at the begining of the line.
 	 */
-	for (prevPtr = NULL, seg2Ptr = index.linePtr->segPtr; 
+	for (prevPtr = NULL, seg2Ptr = index.linePtr->segPtr;
 		seg2Ptr != NULL && seg2Ptr != segPtr;
 		seg2Ptr = seg2Ptr->nextPtr) {
 	    if (seg2Ptr->typePtr == &tkTextRightMarkType ||
@@ -761,7 +765,7 @@ MarkFindPrev(interp, textPtr, string)
 	    }
 	}
 	if (prevPtr != NULL) {
-	    Tcl_SetResult(interp, 
+	    Tcl_SetResult(interp,
 		Tcl_GetHashKey(&textPtr->markTable, prevPtr->body.mark.hPtr),
 		TCL_STATIC);
 	    return TCL_OK;
@@ -773,3 +777,4 @@ MarkFindPrev(interp, textPtr, string)
 	segPtr = NULL;
     }
 }
+
